@@ -484,6 +484,32 @@ handle_encrypted(const MimeMultipartEncrypted& part, Message::Private& info)
 		handle_object(part, res->first, info);
 }
 
+static void
+maybe_handle_pkcs7(const MimeObject& obj, Message::Private& info)
+{
+	if (obj.is_mime_application_pkcs7_mime()) {
+		MimeApplicationPkcs7Mime smime(obj);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wswitch-enum"
+		// CompressedData, CertsOnly, Unknown
+		const auto smtype{smime.smime_type()};
+		switch (smtype) {
+		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::SignedData:
+			info.flags |= Flags::Signed;
+			break;
+		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::EnvelopedData:
+		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::Unknown:
+			/* The "Unknown" case... GMIME doesn't give us anything
+			 * more specific, so assume it's encrypted for now  */
+			info.flags |= Flags::Encrypted;
+			break;
+		default:
+			break;
+		}
+#pragma GCC diagnostic pop
+	}
+}
+
 
 static void
 handle_object(const MimeObject& parent,
@@ -508,23 +534,8 @@ handle_object(const MimeObject& parent,
 		/* FIXME: An encrypted part might be signed at the same time.
 		 *        In that case the signed flag is lost. */
 		info.flags |= Flags::Encrypted;
-	} else if (obj.is_mime_application_pkcs7_mime()) {
-		MimeApplicationPkcs7Mime smime(obj);
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wswitch-enum"
-		// CompressedData, CertsOnly, Unknown
-		switch (smime.smime_type()) {
-		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::SignedData:
-			info.flags |= Flags::Signed;
-			break;
-		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::EnvelopedData:
-			info.flags |= Flags::Encrypted;
-			break;
-		default:
-			break;
-		}
-#pragma GCC diagnostic pop
-	}
+	} else
+		maybe_handle_pkcs7(obj, info);
 }
 
 /**
@@ -548,6 +559,10 @@ process_message(const MimeMessage& mime_msg, const std::string& path,
 			info.flags |= Flags::Unread;
 	}
 
+	// handle top-level
+	if (const auto mpart = mime_msg.mime_part(); mpart)
+		maybe_handle_pkcs7(*mpart, info);
+
 	// parts
 	mime_msg.for_each([&](auto&& parent, auto&& child_obj) {
 		handle_object(parent, child_obj, info);
