@@ -1,6 +1,6 @@
-;;; mu4e-headers.el -- part of mu4e -*- lexical-binding: t; coding:utf-8 -*-
+;;; mu4e-headers.el --- Message headers -*- lexical-binding: t; coding:utf-8 -*-
 
-;; Copyright (C) 2011-2023 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2024 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -35,8 +35,6 @@
 (require 'mule-util) ;; seems _some_ people need this for
                      ;; truncate-string-ellipsis
 
-(require 'mu4e-update)
-
     ;; utility functions
 (require 'mu4e-server)
 (require 'mu4e-vars)
@@ -50,11 +48,11 @@
 (require 'mu4e-lists)
 (require 'mu4e-update)
 (require 'mu4e-folders)
+(require 'mu4e-thread)
 
 (declare-function mu4e-view       "mu4e-view")
 (declare-function mu4e--main-view  "mu4e-main")
 
-
 
 ;;; Configuration
 
@@ -78,15 +76,24 @@ for the rightmost (last) field. Note that emacs may become very
 slow with excessively long lines (1000s of characters), so if you
 regularly get such messages, you want to avoid fields with nil
 altogether."
-  :type `(repeat (cons (choice ,@(mapcar (lambda (h)
-                                           (list 'const :tag
-                                                 (plist-get (cdr h) :help)
-                                                 (car h)))
-                                         mu4e-header-info))
+  :type `(repeat (cons (choice
+                        ,@(mapcar (lambda (h)
+                                    (list 'const :tag
+                                          (plist-get (cdr h) :help)
+                                          (car h)))
+                                  mu4e-header-info)
+                        (restricted-sexp
+                         :tag "User-specified header"
+                         :match-alternatives (mu4e--headers-header-p)))
                        (choice (integer :tag "width")
                                (const :tag "unrestricted width" nil))))
   :group 'mu4e-headers)
 
+(defun mu4e--headers-header-p (symbol)
+  "Is SYMBOL a valid mu4e header?
+This means its either one of the build-in or user-specified headers."
+  (assoc symbol (append mu4e-header-info mu4e-header-info-custom)))
+
 (defcustom mu4e-headers-date-format "%x"
   "Date format to use in the headers view.
 In the format of `format-time-string'."
@@ -105,8 +112,6 @@ In the format of `format-time-string'."
   :type  'string
   :group 'mu4e-headers)
 
-
-
 (defcustom mu4e-headers-precise-alignment nil
   "When set, use precise (but relatively slow) alignment for columns.
 By default, do it in a slightly inaccurate but faster way. To get
@@ -117,13 +122,15 @@ for non-precise aligment (for 445 messages)."
   :group 'mu4e-headers)
 
 (defcustom mu4e-headers-auto-update t
-  "Whether to automatically update the current headers buffer if an
-indexing operation showed changes."
+  "Automatically update the current headers buffer?
+After an indexing operation with changes."
   :type 'boolean
   :group 'mu4e-headers)
 
 (defcustom mu4e-headers-advance-after-mark t
-  "With this option set to non-nil, automatically advance to the
+  "Advance to the next header after marking?
+
+With this option set to non-nil, automatically advance to the
 next mail after marking a message in header view."
   :type 'boolean
   :group 'mu4e-headers)
@@ -154,16 +161,19 @@ mostly covered by `new', and the display gets cluttered otherwise."
   :group 'mu4e-headers)
 
 (defcustom mu4e-headers-found-hook nil
-  "Hook run just *after* all of the headers for the last search
-query have been received and are displayed."
+  "Hook run just after all of the headers have been received.
+I.e., for the last search query have been received and are
+displayed."
   :type 'hook
   :group 'mu4e-headers)
 
 ;;; Public variables
 (defcustom mu4e-headers-from-or-to-prefix '("" . "To ")
-  "Prefix for the :from-or-to field when it is showing,
-  respectively, From: or To:.  It is a cons cell with the car
-  element being the From: prefix, the cdr element the To: prefix."
+  "Prefix for the :from-or-to field.
+
+When it is showing, respectively, From: or To:. It is a cons cell
+  with the car element being the From: prefix, the cdr element
+  the To: prefix."
   :type '(cons string string)
   :group 'mu4e-headers)
 
@@ -238,15 +248,18 @@ Must have the same length as `mu4e-headers-thread-connection-prefix'.")
 
 ;;;; Various
 
-(defvar mu4e-headers-actions
+(defcustom mu4e-headers-actions
   '( ("capture message"  . mu4e-action-capture-message)
+     ("browse online archive" . mu4e-action-browse-list-archive)
      ("show this thread" . mu4e-action-show-thread))
   "List of actions to perform on messages in the headers list.
 The actions are cons-cells of the form (NAME . FUNC) where:
 * NAME is the name of the action (e.g. \"Count lines\")
 * FUNC is a function which receives a message plist as an argument.
 
-The first character of NAME is used as the shortcut.")
+The first character of NAME is used as the shortcut."
+  :group 'mu4e-headers
+  :type '(alist :key-type string :value-type function))
 
 (defvar mu4e-headers-custom-markers
   '(("Older than"
@@ -258,9 +271,10 @@ The first character of NAME is used as the shortcut.")
     ("Bigger than"
      (lambda (msg bytes) (> (mu4e-msg-field msg :size) (* 1024 bytes)))
      (lambda () (read-number "Match messages bigger than (Kbytes): "))))
-  "List of custom markers -- functions to mark message that match
-some custom function. Each of the list members has the following
-format:
+  "List of custom markers-functions.
+
+The functions are to mark message that match some custom
+function. Each of the list members has the following format:
   (NAME PREDICATE-FUNC PARAM-FUNC)
 * NAME is the name of the predicate function, and the first
 character is the shortcut (so keep those unique).
@@ -274,13 +288,14 @@ getting user-input.")
 
 ;; docid cookies
 (defconst mu4e~headers-docid-pre "\376"
-  "Each header starts (invisibly) with the `mu4e~headers-docid-pre',
+  "Prefix for header (before).
+Each header starts (invisibly) with `mu4e~headers-docid-pre',
 followed by the docid, followed by `mu4e~headers-docid-post'.")
 (defconst mu4e~headers-docid-post "\377"
-  "Each header starts (invisibly) with the `mu4e~headers-docid-pre',
+  "Prefix for header (after).
+Each header starts (invisibly) with the `mu4e~headers-docid-pre',
 followed by the docid, followed by `mu4e~headers-docid-post'.")
 
-
 (defvar mu4e~headers-search-start nil)
 (defvar mu4e~headers-render-start nil)
 (defvar mu4e~headers-render-time  nil)
@@ -292,7 +307,6 @@ This is mostly useful for profiling.")
 (defvar mu4e~headers-hidden 0
   "Number of headers hidden due to `mu4e-headers-hide-predicate'.")
 
-
 
 ;;; Clear
 
@@ -305,12 +319,12 @@ Optionally, show TEXT."
     (let ((inhibit-read-only t))
       (with-current-buffer (mu4e-get-headers-buffer)
         (mu4e--mark-clear)
+        (remove-overlays)
         (erase-buffer)
         (when text
           (goto-char (point-min))
           (insert (propertize text 'face 'mu4e-system-face 'intangible t)))))))
 
-
 ;;; Misc
 
 (defun mu4e~headers-contact-str (contacts)
@@ -327,17 +341,19 @@ into a string."
   (let ((get-prefix
          (lambda (cell)
            (if mu4e-use-fancy-chars (cdr cell) (car cell)))))
-    (cl-case type
-      (child         (funcall get-prefix mu4e-headers-thread-child-prefix))
-      (first-child   (funcall get-prefix mu4e-headers-thread-first-child-prefix))
-      (last-child    (funcall get-prefix mu4e-headers-thread-last-child-prefix))
-      (connection    (funcall get-prefix mu4e-headers-thread-connection-prefix))
-      (blank         (funcall get-prefix mu4e-headers-thread-blank-prefix))
-      (orphan        (funcall get-prefix mu4e-headers-thread-orphan-prefix))
-      (single-orphan (funcall get-prefix
-                              mu4e-headers-thread-single-orphan-prefix))
-      (duplicate     (funcall get-prefix mu4e-headers-thread-duplicate-prefix))
-      (t              "?"))))
+    (propertize
+     (cl-case type
+       (child         (funcall get-prefix mu4e-headers-thread-child-prefix))
+       (first-child   (funcall get-prefix mu4e-headers-thread-first-child-prefix))
+       (last-child    (funcall get-prefix mu4e-headers-thread-last-child-prefix))
+       (connection    (funcall get-prefix mu4e-headers-thread-connection-prefix))
+       (blank         (funcall get-prefix mu4e-headers-thread-blank-prefix))
+       (orphan        (funcall get-prefix mu4e-headers-thread-orphan-prefix))
+       (single-orphan (funcall get-prefix
+                               mu4e-headers-thread-single-orphan-prefix))
+       (duplicate     (funcall get-prefix mu4e-headers-thread-duplicate-prefix))
+       (t              "?"))
+     'face 'mu4e-thread-fold-face)))
 
 
 ;; headers in the buffer are prefixed by an invisible string with the docid
@@ -347,23 +363,24 @@ into a string."
 ;; docid at point without string matching etc.
 
 (defun mu4e~headers-docid-pos (docid)
-  "Return the pos of the beginning of the line with the header with
-docid DOCID, or nil if it cannot be found."
+  "Return beginning-of-line for header with DOCID.
+Return nil if it cannot be found."
   (let ((pos))
     (save-excursion
       (setq pos (mu4e~headers-goto-docid docid)))
     pos))
 
 (defun mu4e~headers-docid-cookie (docid)
-  "Create an invisible string containing DOCID; this is to be used
-at the beginning of lines to identify headers."
+  "Create an invisible string containing DOCID.
+
+This is to be used at the beginning of lines to identify headers."
   (propertize (format "%s%d%s"
                       mu4e~headers-docid-pre docid mu4e~headers-docid-post)
               'docid docid 'invisible t));;
 
 (defun mu4e~headers-docid-at-point (&optional point)
-  "Get the docid for the header at POINT, or at current (point) if
-nil. Returns the docid, or nil if there is none."
+  "Get docid for header at POINT or (point).
+Returns the docid, or nil if there is none."
   (save-excursion
     (when point
       (goto-char point))
@@ -372,8 +389,8 @@ nil. Returns the docid, or nil if there is none."
 
 
 (defun mu4e~headers-goto-docid (docid &optional to-mark)
-  "Go to the beginning of the line with the header with docid
-DOCID, or nil if it cannot be found. If the optional TO-MARK is
+  "Go to beginning of the line with DOCID.
+Nil if it cannot be found. If the optional TO-MARK is
 non-nil, go to the point directly *after* the docid-cookie instead
 of the beginning of the line."
   (let ((oldpoint (point)) (newpoint))
@@ -389,8 +406,7 @@ of the beginning of the line."
     newpoint)) ;; return the point, or nil if not found
 
 (defun mu4e~headers-field-for-docid (docid field)
-  "Get FIELD (a symbol, see `mu4e-headers-names') for the message
-with DOCID which must be present in the headers buffer."
+  "Get FIELD for the message with DOCID, present in the headers buffer."
   (save-excursion
     (when (mu4e~headers-goto-docid docid)
       (mu4e-message-field (mu4e-message-at-point) field))))
@@ -526,14 +542,13 @@ e.g. \"mutt\"."
          (truncate-string-to-width subj 600) ""))))
 
 (defun mu4e~headers-mailing-list (list)
-  "Get some identifier for the mailing list."
+  "Get some identifier for mailing LIST."
   (if list
       (propertize (mu4e-get-mailing-list-shortname list) 'help-echo list)
     ""))
 
 (defsubst mu4e~headers-custom-field-value (msg field)
-  "Show some custom header field, or raise an error if it is not
-found."
+  "Show custom header FIELD for MSG, or raise error if not found."
   (let* ((item (or (assoc field mu4e-header-info-custom)
                    (mu4e-error "field %S not found" field)))
          (func (or (plist-get (cdr-safe item) :function)
@@ -542,6 +557,7 @@ found."
     (funcall func msg)))
 
 (defun mu4e~headers-field-value (msg field)
+  "Get the value for MSG FIELD."
   (let ((val (mu4e-message-field msg field)))
     (cl-case field
       (:subject
@@ -580,9 +596,11 @@ found."
     val))
 
 (defun mu4e~headers-truncate-field-precise (field val width)
-  "Return VAL truncated to one less than WIDTH, with a trailing
+  "Truncate FIELD value VAL to WIDTH precisely.
+
+Return VAL truncated to one less than WIDTH, with a trailing
 space propertized with a `display' text property which expands to
- the correct column for display."
+the correct column for display."
   (when width
     (let ((end-col (cl-loop for (f . w) in mu4e-headers-fields
                             sum w
@@ -602,7 +620,7 @@ space propertized with a `display' text property which expands to
   val)
 
 (defsubst mu4e~headers-truncate-field (field val width)
-  "Truncate VAL to WIDTH."
+  "Truncate FIELD value VAL to WIDTH."
   (if mu4e-headers-precise-alignment
       (mu4e~headers-truncate-field-precise field val width)
     (mu4e~headers-truncate-field-fast val width)))
@@ -636,8 +654,8 @@ space propertized with a `display' text property which expands to
     fieldval))
 
 (defsubst mu4e~message-header-line (msg)
-  "Return a propertized description of message MSG suitable for
-displaying in the header view."
+  "Return a propertized description of MSG.
+This is suitable for displaying in the header view."
   (if (and mu4e-search-hide-enabled mu4e-search-hide-predicate
            (funcall mu4e-search-hide-predicate msg))
       (progn
@@ -651,7 +669,7 @@ displaying in the header view."
 
 (defsubst mu4e~headers-insert-header (msg pos)
   "Insert a header for MSG at point POS."
-  (when-let ((line (mu4e~message-header-line msg))
+  (when-let* ((line (mu4e~message-header-line msg))
              (docid (plist-get msg :docid)))
     (goto-char pos)
     (insert
@@ -663,7 +681,7 @@ displaying in the header view."
 
 (defun mu4e~headers-remove-header (docid &optional ignore-missing)
   "Remove header with DOCID at point.
-When IGNORE-MISSING is non-nill, don't raise an error when the
+When IGNORE-MISSING is non-nil, don't raise an error when the
 docid is not found."
   (with-current-buffer (mu4e-get-headers-buffer)
     (if (mu4e~headers-goto-docid docid)
@@ -672,14 +690,14 @@ docid is not found."
       (unless ignore-missing
         (mu4e-error "Cannot find message with docid %S" docid)))))
 
-
+
 ;;; Handler functions
 
 ;; next are a bunch of handler functions; those will be called from mu4e~proc in
 ;; response to output from the server process
 
 (defun mu4e~headers-view-handler (msg)
-  "Handler function for displaying a message."
+  "Handler function for displaying a MSG."
   (mu4e-view msg))
 
 (defun mu4e~headers-view-this-message-p (docid)
@@ -689,7 +707,7 @@ docid is not found."
 
 ;; note: this function is very performance-sensitive
 (defun mu4e~headers-append-handler (msglst)
-  "Append one-line descriptions of messages in MSGLIST.
+  "Append one-line descriptions of messages in MSGLST.
 Do this at the end of the headers-buffer."
   (when (buffer-live-p (mu4e-get-headers-buffer))
     (with-current-buffer (mu4e-get-headers-buffer)
@@ -702,9 +720,10 @@ Do this at the end of the headers-buffer."
 
 
 (defun mu4e~headers-update-handler (msg is-move maybe-view)
-  "Update handler, will be called when a message has been updated
-in the database. This function will update the current list of
-headers."
+  "Update handler, called when MSG is updated.
+This function will update the current list of headers
+IS-MOVE specifies if this was a move and MAYBE-VIEW whether
+the updated message should be viewed."
   (when (buffer-live-p (mu4e-get-headers-buffer))
     (with-current-buffer (mu4e-get-headers-buffer)
       (let* ((docid (mu4e-message-field msg :docid))
@@ -758,8 +777,9 @@ headers."
           (run-hooks 'mu4e-message-changed-hook))))))
 
 (defun mu4e~headers-remove-handler (docid)
-  "Remove handler, will be called when a message with DOCID has
-been removed from the database. This function will hide the removed
+  "Remove handler, called when a message with DOCID is removed.
+
+This function will hide the removed
 message from the current list of headers. If the message is not
 present, don't do anything."
   (when (buffer-live-p (mu4e-get-headers-buffer))
@@ -773,12 +793,17 @@ present, don't do anything."
       (kill-buffer buf))))
 
 
-
+
 ;;; Performing queries (internal)
 (defconst mu4e~search-message "Searching...")
 (defconst mu4e~no-matches     "No matching messages found")
 (defconst mu4e~end-of-results "End of search results")
 
+(defvar mu4e--search-background nil
+  "Is this a background search?
+If so, do not attempt to switch buffers. This variable is to be let-bound
+to t before \"automatic\" searches.")
+
 (defun mu4e--search-execute (expr ignore-history)
   "Search for query EXPR.
 
@@ -806,7 +831,7 @@ true, do *not* update the query history stack."
 
     ;; when the buffer is already visible, select it; otherwise,
     ;; switch to it.
-    (unless (get-buffer-window buf 0)
+    (unless (get-buffer-window buf (if mu4e--search-background 0 nil))
       (mu4e-display-buffer buf t))
     (run-hook-with-args 'mu4e-search-hook expr)
     (mu4e~headers-clear mu4e~search-message)
@@ -836,8 +861,7 @@ true, do *not* update the query history stack."
     ""))
 
 (defun mu4e~headers-found-handler (count)
-  "Create a one line description of the number of headers found
-after the end of the search results."
+  "Create one=line description of the COUNT of headers found."
   (when (buffer-live-p (mu4e-get-headers-buffer))
     (with-current-buffer (mu4e-get-headers-buffer)
       (save-excursion
@@ -853,30 +877,33 @@ after the end of the search results."
           (unless (zerop count)
             (mu4e-message "%s" msg))))
 
-          ;; if we need to jump to some specific message, do so now
-          (goto-char (point-min))
-          (when mu4e--search-msgid-target
-            (if (eq (current-buffer) (window-buffer))
-                (mu4e-headers-goto-message-id mu4e--search-msgid-target)
-              (let* ((pos (mu4e-headers-goto-message-id
-                           mu4e--search-msgid-target)))
-                (when pos
-                  (set-window-point (get-buffer-window nil t) pos)))))
-          (when (and mu4e--search-view-target (mu4e-message-at-point 'noerror))
-            ;; view the message at point when there is one.
-            (mu4e-headers-view-message))
-          (setq mu4e--search-view-target nil
-                mu4e--search-msgid-target nil)
-          (when (mu4e~headers-docid-at-point)
-            (mu4e~headers-highlight (mu4e~headers-docid-at-point)))))
-    ;; run-hooks
-    (run-hooks 'mu4e-headers-found-hook))
+      ;; if we need to jump to some specific message, do so now
+      (goto-char (point-min))
+      (when mu4e--search-msgid-target
+        (if (eq (current-buffer) (window-buffer))
+            (mu4e-headers-goto-message-id mu4e--search-msgid-target)
+          (let* ((pos (mu4e-headers-goto-message-id
+                       mu4e--search-msgid-target)))
+            (when pos
+              (set-window-point (get-buffer-window nil t) pos)))))
+      (when (and mu4e--search-view-target (mu4e-message-at-point 'noerror))
+        ;; view the message at point when there is one.
+        (mu4e-headers-view-message))
+      (setq mu4e--search-view-target nil
+            mu4e--search-msgid-target nil)
+      (when (mu4e~headers-docid-at-point)
+        (mu4e~headers-highlight (mu4e~headers-docid-at-point)))
+      ;; maybe enable thread folding
+      (when mu4e-search-threads
+        (mu4e-thread-mode))))
+  ;; run-hooks
+  (run-hooks 'mu4e-headers-found-hook))
+
 
-
 ;;; Marking
 
 (defmacro mu4e~headers-defun-mark-for (mark)
-  "Define a function mu4e~headers-mark-MARK."
+  "Define a function mu4e~headers-mark-... for MARK."
   (let ((funcname (intern (format "mu4e-headers-mark-for-%s" mark)))
         (docstring (format "Mark header at point with %s." mark)))
     `(progn
@@ -972,10 +999,6 @@ after the end of the search results."
     (define-key map "a" #'mu4e-headers-action)
 
     ;; message composition
-    (define-key map "R" #'mu4e-compose-reply)
-    (define-key map "F" #'mu4e-compose-forward)
-    (define-key map "C" #'mu4e-compose-new)
-    (define-key map "E" #'mu4e-compose-edit)
 
     (define-key map (kbd "RET") #'mu4e-headers-view-message)
     (define-key map [mouse-2]   #'mu4e-headers-view-message)
@@ -994,7 +1017,7 @@ after the end of the search results."
      "--"
      ["Previous" mu4e-headers-prev
       :help "Move to previous header"]
-     ["Next" mu4e-headers-prev
+     ["Next" mu4e-headers-next
       :help "Move to next header"]
      "--"
      ["Mark for move" mu4e-headers-mark-for-move
@@ -1069,20 +1092,26 @@ after the end of the search results."
       mu4e-headers-fields))))
 
 (defun mu4e~headers-maybe-auto-update ()
-  "Update the current headers buffer after indexing has brought
-some changes, `mu4e-headers-auto-update' is non-nil and there is
-no user-interaction ongoing."
-  (when (and mu4e-headers-auto-update          ;; must be set
-             mu4e-index-update-status
-             (not (zerop (plist-get mu4e-index-update-status :updated)))
-             ;; NOTE: `mu4e-mark-marks-num' can return nil. Is that intended?
-             (zerop (or (mu4e-mark-marks-num) 0))     ;; non active marks
-             (not (active-minibuffer-window))) ;; no user input only
-    ;; rerun search if there's a live window with search results;
-    ;; otherwise we'd trigger a headers view from out of nowhere.
-    (when (and (buffer-live-p (mu4e-get-headers-buffer))
-               (window-live-p (get-buffer-window (mu4e-get-headers-buffer) t)))
-      (mu4e-search-rerun))))
+  "Update the current headers buffer after indexing changes.
+
+Furthermore, `mu4e-headers-auto-update' is non-nil and there is
+no user-interaction ongoing.
+
+We only update headers when quite a few conditions are true --
+see the code."
+  (when-let* ((hdrsbuf (mu4e-get-headers-buffer)))
+    (when (and mu4e-headers-auto-update ;; must be set
+               mu4e-index-update-status
+               (not (mu4e-get-view-buffer)) ;; not when viewing a message
+               (not (zerop (plist-get mu4e-index-update-status :updated)))
+               (buffer-live-p hdrsbuf)
+               (window-live-p (get-buffer-window hdrsbuf t))
+               ;; don't disturb marks.
+               (zerop (or (with-current-buffer hdrsbuf (mu4e-mark-marks-num)) 0))
+               (not (active-minibuffer-window))) ;; no user input only
+      ;; when all that is true, rerun the current query.
+      (let ((mu4e--search-background t))
+        (mu4e-search-rerun)))))
 
 (defcustom mu4e-headers-eldoc-format "“%s” from %f on %d"
   "Format for the `eldoc' string for the current message in the headers buffer.
@@ -1100,6 +1129,7 @@ The following specs are supported:
   :group 'mu4e-headers)
 
 (defun mu4e-headers-eldoc-function (&rest _args)
+  "Determine function for eldoc."
   (let ((msg (get-text-property (point) 'msg)))
     (when msg
       (format-spec
@@ -1148,6 +1178,7 @@ The following specs are supported:
   (mu4e-context-minor-mode)
   (mu4e-update-minor-mode)
   (mu4e-search-minor-mode)
+  (mu4e-compose-minor-mode)
   (hl-line-mode 1)
 
   (mu4e--modeline-register #'mu4e--search-modeline-item)
@@ -1156,11 +1187,11 @@ The following specs are supported:
 ;;; Highlighting
 
 (defvar mu4e~highlighted-docid nil
-  "The highlighted docid")
+  "The highlighted docid.")
 
 (defun mu4e~headers-highlight (docid)
   "Highlight the header with DOCID, or do nothing if it's not found.
-Also, unhighlight any previously highlighted headers."
+Also, un-highlight any previously highlighted headers."
   (with-current-buffer (mu4e-get-headers-buffer)
     (save-excursion
       ;; first, unhighlight the previously highlighted docid, if any
@@ -1175,15 +1206,16 @@ Also, unhighlight any previously highlighted headers."
 ;;; Misc 2
 
 (defun mu4e~headers-select-window ()
-  "When there is a visible window for the headers buffer, make sure
-to select it. This is needed when adding new headers, otherwise
+  "Select a visible window for the headers buffer if it exists.
+
+This is needed when adding new headers, otherwise
 adding a lot of new headers looks really choppy."
   (let ((win (get-buffer-window (mu4e-get-headers-buffer))))
     (when win (select-window win))))
 
 (defun mu4e-headers-goto-message-id (msgid)
-  "Go to the next message with message-id MSGID. Return the
-message plist, or nil if not found."
+  "Go to the next message with MSGID.
+Return the message plist, or nil if not found."
   (mu4e-headers-find-if
    (lambda (msg)
      (let ((this-msgid (mu4e-message-field msg :message-id)))
@@ -1214,7 +1246,6 @@ message plist, or nil if not found."
                  'msg msg)))
       (goto-char oldpoint))))
 
-
 ;;; Queries & searching
 
 ;;; Search-based marking
@@ -1232,13 +1263,18 @@ corresponding header."
         (when msg
           (funcall func msg))))))
 
+
 (defun mu4e-headers-find-if (func &optional backward)
-  "Move to the next header for which FUNC returns non-`nil',
-starting from the current position. FUNC receives one argument, the
-message s-expression for the corresponding header. If BACKWARD is
-non-`nil', search backwards. Returns the new position, or `nil' if
-nothing was found. If you want to exclude matches for the current
-message, you can use `mu4e-headers-find-if-next'."
+  "Move to the header for which FUNC returns non-nil.
+if BACKWARD is non-nil, search backwards.
+
+FUNC receives one argument, the message s-expression for the
+corresponding header. If BACKWARD is non-nil, search backwards.
+Returns the new position, or nil if nothing was found. If you
+want to exclude matches for the current message, you can use
+`mu4e-headers-find-if-next'.
+
+Return the found position or nil if not found."
   (let ((pos)
         (search-func (if backward 'search-backward 'search-forward)))
     (save-excursion
@@ -1250,17 +1286,17 @@ message, you can use `mu4e-headers-find-if-next'."
           (when (and msg (funcall func msg))
             (setq pos (point))))))
     (when pos
-      (goto-char pos))))
+      (goto-char pos))
+    pos))
 
 (defun mu4e-headers-find-if-next (func &optional backwards)
   "Like `mu4e-headers-find-if', but do not match the current header.
-Move to the next or (if BACKWARDS is non-`nil') header for which FUNC
-returns non-`nil', starting from the current position."
+
+Move to the next or (if BACKWARDS is non-nil) header for which
+FUNC returns non-nil, starting from the current position."
   (let ((pos))
     (save-excursion
-      (if backwards
-          (beginning-of-line)
-        (end-of-line))
+      (if backwards (beginning-of-line) (end-of-line))
       (setq pos (mu4e-headers-find-if func backwards)))
     (when pos (goto-char pos))))
 
@@ -1268,8 +1304,8 @@ returns non-`nil', starting from the current position."
   "History list of regexps used.")
 
 (defun mu4e-headers-mark-for-each-if (markpair mark-pred &optional param)
-  "Mark all headers for which predicate function MARK-PRED returns
-non-nil with MARKPAIR. MARK-PRED is function that receives two
+  "Mark headers with predicate MARK-PRED with MARKPAIR.
+MARK-PRED is function that receives two
 arguments, MSG (the message at point) and PARAM (a user-specified
 parameter). MARKPAIR is a cell (MARK . TARGET); see
 `mu4e-mark-at-point' for details about marks."
@@ -1279,9 +1315,11 @@ parameter). MARKPAIR is a cell (MARK . TARGET); see
        (mu4e-mark-at-point (car markpair) (cdr markpair))))))
 
 (defun mu4e-headers-mark-pattern ()
-  "Ask user for a kind of mark (move, delete etc.), a field to
-match and a regular expression to match with. Then, mark all
-matching messages with that mark."
+  "Ask user for a kind of mark-pattern.
+
+That is, some markpair (move, delete etc.), a field to match and
+a regular expression to match with. Then, mark all matching
+messages with that mark."
   (interactive)
   (let ((markpair (mu4e--mark-get-markpair "Mark matched messages with: " t))
         (field (mu4e-read-option "Field to match: "
@@ -1332,9 +1370,9 @@ matching messages with that mark."
       (otherwise (mu4e-error "Not supported")))))
 
 (defun mu4e-headers-mark-thread-using-markpair (markpair &optional subthread)
-  "Mark the thread at point using the given markpair. If SUBTHREAD is
-non-nil, marking is limited to the message at point and its
-descendants."
+  "Mark the thread at point using the given MARKPAIR.
+If SUBTHREAD is non-nil, marking is limited to the message at
+point and its descendants."
   (let* ((mark (car markpair))
          (allowed-marks (mapcar 'car mu4e-marks)))
     (unless (memq mark allowed-marks)
@@ -1367,7 +1405,9 @@ descendants."
       (mu4e-headers-next))))
 
 (defun mu4e-headers-mark-thread (&optional subthread markpair)
-  "Like `mu4e-headers-mark-thread-using-markpair' but prompt for the markpair."
+  "Like `mu4e-headers-mark-thread-using-markpair' but prompt for MARKPAIR.
+
+If SUBTHREAD is non-nil, only apply to subthread."
   (interactive
    (let* ((subthread current-prefix-arg))
      (list current-prefix-arg
@@ -1379,14 +1419,12 @@ descendants."
   (mu4e-headers-mark-thread-using-markpair markpair subthread))
 
 (defun mu4e-headers-mark-subthread (&optional markpair)
-  "Like `mu4e-mark-thread', but only for a sub-thread."
+  "Like `mu4e-mark-thread' with MARKPAIR, but only for a sub-thread."
   (interactive)
   (if markpair (mu4e-headers-mark-thread t markpair)
     (let ((current-prefix-arg t))
       (call-interactively 'mu4e-headers-mark-thread))))
 
-
-
 (defun mu4e-headers-view-message ()
   "View message at point."
   (interactive)
@@ -1402,60 +1440,70 @@ descendants."
           (if (functionp mu4e-view-auto-mark-as-read)
               (funcall mu4e-view-auto-mark-as-read msg)
             mu4e-view-auto-mark-as-read)))
-    (when-let ((buf (mu4e-get-view-buffer (current-buffer) nil)))
+    (when-let* ((buf (mu4e-get-view-buffer (current-buffer) nil)))
       (with-current-buffer buf
         (mu4e-loading-mode 1)))
     (mu4e--server-view docid mark-as-read)))
 
+(defvar-local mu4e-headers-open-after-move t
+  "Influence open behavior after moving.
+
+If set to non-nil, open message after `mu4e-headers-next' and
+`mu4e-headers-prev' if pointing at a message after the move and
+there is a live message view.
+
+This variable is for let-binding when scripting.")
 
 (defun mu4e~headers-move (lines)
-  "Move point LINES lines.
-Move foward if LINES is positive or backwards if LINES is
+    "Move point LINES lines.
+Move forward if LINES is positive or backwards if LINES is
 negative. If this succeeds, return the new docid. Otherwise,
-return nil."
-  (unless (eq major-mode 'mu4e-headers-mode)
-    (mu4e-error "Must be in mu4e-headers-mode (%S)" major-mode))
-  (cl-flet ((goto-next-line
-              (arg)
-              (condition-case _err
-                  (prog1
-                      (let (line-move-visual)
-                        (and (line-move arg) 0))
-                    ;; Skip invisible text at BOL possibly hidden by
-                    ;; the end of another invisible overlay covering
-                    ;; previous EOL.
-                    (move-to-column 2))
-                ((beginning-of-buffer end-of-buffer)
-                 1))))
-    (let* ((succeeded (zerop (goto-next-line lines)))
-           (docid (mu4e~headers-docid-at-point)))
-      ;; move point, even if this function is called when this window is not
-      ;; visible
-      (when docid
-        ;; update all windows showing the headers buffer
-        (walk-windows
-         (lambda (win)
-           (when (eq (window-buffer win)
-                     (mu4e-get-headers-buffer (buffer-name)))
-             (set-window-point win (point))))
-         nil t)
-        ;; If the assigned (and buffer-local) `mu4e~headers-view-win'
-        ;; is not live then that is indicates the user does not want
-        ;; to pop up the view when they navigate in the headers
-        ;; buffer.
-        (when (window-live-p mu4e~headers-view-win)
-          (mu4e-headers-view-message))
-        ;; attempt to highlight the new line, display the message
-        (mu4e~headers-highlight docid)
-        (if succeeded
-            docid
-          nil)))))
+return nil.
+
+If pointing at a message after the move and there is a
+view-window, open the message unless
+`mu4e-headers-open-after-move' is non-nil."
+  (cl-assert (eq major-mode 'mu4e-headers-mode))
+  (when (ignore-errors
+          (let (line-move-visual)
+            (line-move lines)
+            t))
+    (let* ((docid (mu4e~headers-docid-at-point))
+           (folded (and docid (mu4e-thread-message-folded-p))))
+      (if folded
+          (mu4e~headers-move (if (< lines 0) -1 1)) ;; skip folded
+        (when docid
+          ;; Skip invisible text at BOL possibly hidden by
+          ;; the end of another invisible overlay covering
+          ;; previous EOL.
+          (move-to-column 2)
+          ;; update all windows showing the headers buffer
+          (walk-windows
+           (lambda (win)
+             (when (eq (window-buffer win)
+                       (mu4e-get-headers-buffer (buffer-name)))
+               (set-window-point win (point))))
+           nil t)
+          ;; If the assigned (and buffer-local) `mu4e~headers-view-win'
+          ;; is not live then that is indicates the user does not want
+          ;; to pop up the view when they navigate in the headers
+          ;; buffer.
+          (when (and mu4e-headers-open-after-move
+                     (window-live-p mu4e~headers-view-win))
+            (mu4e-headers-view-message))
+          ;; attempt to highlight the new line, display the message
+          (mu4e~headers-highlight docid)
+          docid)))))
 
 (defun mu4e-headers-next (&optional n)
   "Move point to the next message header.
 If this succeeds, return the new docid. Otherwise, return nil.
 Optionally, takes an integer N (prefix argument), to the Nth next
-header."
+header.
+
+If pointing at a message after the move and there is a
+view-window, open the message unless
+`mu4e-headers-open-after-move' is non-nil."
   (interactive "P")
   (mu4e~headers-move (or n 1)))
 
@@ -1463,13 +1511,17 @@ header."
   "Move point to the previous message header.
 If this succeeds, return the new docid. Otherwise, return nil.
 Optionally, takes an integer N (prefix argument), to the Nth
-previous header."
+previous header.
+
+If pointing at a message after the move and there is a
+view-window, open the message unless
+`mu4e-headers-open-after-move' is non-nil."
   (interactive "P")
   (mu4e~headers-move (- (or n 1))))
 
 (defun mu4e~headers-prev-or-next-unread (backwards)
-  "Move point to the next message that is unread (and
-untrashed). If BACKWARDS is non-`nil', move backwards."
+  "Move point to next message that is unread/untrashed.
+If BACKWARDS is non-nil, move backwards."
   (interactive "P")
   (or (mu4e-headers-find-if-next
        (lambda (msg)
@@ -1480,28 +1532,30 @@ untrashed). If BACKWARDS is non-`nil', move backwards."
                             (if backwards "previous" "next")))))
 
 (defun mu4e-headers-prev-unread ()
-  "Move point to the previous message that is unread (and
-untrashed)."
+  "Move point to previous unread/untrashed message."
   (interactive)
   (mu4e~headers-prev-or-next-unread t))
 
 (defun mu4e-headers-next-unread ()
-  "Move point to the next message that is unread (and
-untrashed)."
+  "Move point to next unread/untrashed message."
   (interactive)
   (mu4e~headers-prev-or-next-unread nil))
 
+(defun mu4e~headers-thread-root-p (&optional msg)
+  "Is MSG at the root of a thread?
+If MSG is nil, use message at point."
+  (when-let* ((msg (or msg (get-text-property (point) 'msg)))
+              (meta (mu4e-message-field msg :meta)))
+    (let* ((orphan (plist-get meta :orphan))
+           (first-child (plist-get meta :first-child))
+           (root (plist-get meta :root)))
+      (or root (and orphan first-child)))))
 
 (defun mu4e~headers-prev-or-next-thread (backwards)
   "Move point to the top of the next thread.
-If BACKWARDS is non-`nil', move backwards."
-  (interactive "P")
-  (or (mu4e-headers-find-if-next
-       (lambda (msg)
-         (eq 0 (plist-get (plist-get msg :meta) :level)))
-       backwards)
-      (mu4e-message (format "No %s thread found"
-                            (if backwards "previous" "next")))))
+If BACKWARDS is non-nil, move backwards."
+  (when (mu4e-headers-find-if-next #'mu4e~headers-thread-root-p backwards)
+      (point)))
 
 (defun mu4e-headers-prev-thread ()
   "Move point to the previous thread."
@@ -1552,9 +1606,11 @@ argument."
     (funcall afunc msg)))
 
 (defun mu4e-headers-mark-and-next (mark)
-  "Set mark MARK on the message at point or on all messages in the
-region if there is a region, then move to the next message."
+  "Set MARK on the message at point or in region.
+Then, move to the next message."
   (interactive)
+  (when (mu4e-thread-message-folded-p)
+    (mu4e-warn "Cannot mark folded messages"))
   (mu4e-mark-set mark)
   (when mu4e-headers-advance-after-mark (mu4e-headers-next)))
 
@@ -1567,11 +1623,9 @@ region if there is a region, then move to the next message."
   (mu4e--query-items-refresh 'reset-baseline)
   (mu4e--main-view))
 
-
 ;;; Loading messages
 ;;
 
-
 (defvar-local mu4e--loading-overlay-bg nil
   "Internal variable that holds the loading overlay for the background.")
 
@@ -1579,7 +1633,7 @@ region if there is a region, then move to the next message."
   "Internal variable that holds the loading overlay for the text.")
 
 (define-minor-mode mu4e-loading-mode
-  "Minor mode for buffers awaiting data from mu"
+  "Minor mode for buffers awaiting data from mu."
   :init-value nil :lighter " Loading" :keymap nil
   (if mu4e-loading-mode
       (progn
